@@ -17,7 +17,7 @@ class CFC(object):
             http://zone.ni.com/reference/en-XX/help/370859J-01/crash/misc_cfc/
         """
 
-        assert(cfc in [60, 180, 600, 1000])
+        assert(cfc in [13, 16, 60, 180, 600, 1000])
 
         wd = 2. * np.pi * cfc * 2.0775
         wa = np.sin(wd * T / 2.) / np.cos(wd * T / 2.)
@@ -53,7 +53,7 @@ class CFC(object):
         # 1. pre-event data - mirror data on point of origin
         duplicated = np.transpose(np.transpose(sampled_array[:, 1:start_index])[::-1]) * -1
         if mirroring_flag:
-            mirroring = np.transpose(np.transpose(sampled_array[:, 1:start_index + 1])[::])
+            mirroring = np.transpose(np.transpose(sampled_array[:, 1:start_index + 2])[::])
             temp = np.zeros((1, start_index))
             for index, value in enumerate(mirroring[0]):
                 if index >= start_index:
