@@ -26,10 +26,17 @@ class StandardFunction:
     @staticmethod
     def max(data_vector: np.ndarray, units):
         """
-        :param data_vector:
-        :param absolute:
-        :return:
+        Return the maximum of an array. 
+        Further info : https://numpy.org/doc/stable/reference/generated/numpy.amax.html
+        :param data_vector: The data_vector from which the maximum value should be 
+        :type data_vector: np.ndarray
+        :param units: Units object, information about the used system of units in the simulated output
+        :type units: Units as implemented in /dynasaur/data/dynasaur_definitions.py
+
+        :returns: Maximum of data_vector
+        :rtype: scalar
         """
+
         return np.max(data_vector)
 
     @staticmethod
@@ -303,10 +310,10 @@ class StandardFunction:
             units):
         """
 
-        :param time
+        :param time_
         :param acc_x:
         :param acc_y:
-        :param rotational_velocity_z:
+        :param rvel_z:
         :param Dx:
         :param Dy:
         :param units:
