@@ -1,5 +1,6 @@
 
-from ..data.ls_dyna import DEForc, Elout, EloutObject, Glstat, Matsum, Nodout, RCForc, SBTout, Secforc, Disbout, PartDisbout, Abstat, AbstatCPM, JointForc, Sleout
+from ..data.ls_dyna import DEForc, Elout, EloutObject, Glstat, Matsum, Nodout, RCForc, SBTout, Secforc, Disbout, \
+                           PartDisbout, Abstat, AbstatCPM, JointForc, Sleout, RigidBody, BoundaryCondition
 from ..utils.constants import ObjectConstantsForData
 
 
@@ -24,6 +25,8 @@ class DataContainer:
     _abstat = None
     _abstat_cpm = None
     _sleout = None
+    _rigid_body = None
+    _boundary_condition = None
 
     @staticmethod
     def get_data(name):
@@ -62,7 +65,12 @@ class DataContainer:
             return DataContainer._abstat_cpm
         elif name == ObjectConstantsForData.SLEOUT:
             return DataContainer._sleout
-        return
+        elif name == ObjectConstantsForData.RIGID_BODY:
+            return DataContainer._rigid_body
+        elif name == ObjectConstantsForData.BOUNDARY_CONDITION:
+            return DataContainer._boundary_condition
+        else:
+            assert False
 
     @staticmethod
     def init_all_data_sources(binout, logger, dynasaur_def, volume_path):
@@ -90,4 +98,6 @@ class DataContainer:
         DataContainer._abstat = Abstat(binout, logger, dynasaur_def)
         DataContainer._abstat_cpm = AbstatCPM(binout, logger, dynasaur_def)
         DataContainer._sleout = Sleout(binout, logger, dynasaur_def)
+        DataContainer._rigid_body = RigidBody(binout, logger, dynasaur_def)
+        DataContainer._boundary_condition = BoundaryCondition(binout, logger, dynasaur_def)
 
