@@ -1,10 +1,7 @@
-import functools
-
 import numpy as np
-
-from ..utils.constants import LOGConstants, TestConstants, JsonConstants
 import copy
-import lasso
+
+from ..utils.constants import LOGConstants, JsonConstants
 
 
 class BinoutData(object):
@@ -12,10 +9,15 @@ class BinoutData(object):
     def __init__(self, binout, logger, dynasaur_definitions, name):
         """
         Initialization/constructor
+        1 ) initializes members
+
+        2 ) initialize the member required_data_channel_ids. List of all data_channel_ids in the calc_proc.def file ,
+          i.e. {"type":"ELEMENT","ID":"id1","array":["(0, axial)"]
+                    used to check if channel "axial" is part of type ELEMENT
 
         :param: binout
         :param: logger
-        :param: dynasaur definition
+        :param: dynasaur definition object
         :param: name (binout name)
 
         :return:
@@ -29,41 +31,59 @@ class BinoutData(object):
         self._time_interp = None
         self._data = {}
         self._ids = None
+        self._required_data_channel_ids = []
 
-        self._defined_types = []
-        self._set_defined_types(self._dynasaur_definitions)
-        self._clean_defined_types()
-        self._check_for_id_in_binout()
+        self._set_required_data_channel_ids()
+        self._clean_required_data_channel_ids()
 
     def get_time(self):
         """
-        :return: time array
+        getter
+
+        :returns: time column vector
+        :rtype: np.array
         """
         return self._time
 
     def get_interpolated_time(self):
         """
-        :return: interpolated time array
+        getter
+
+        :returns: interpolated time as column vector
+        :rtype: np.array
         """
         return self._time_interp
 
     def get_channels_ids_object_name(self, object_name, plugin_name):
         """
-        :return: time array
+        :param object_name:
+        :type object_name:
+        :param plugin_name:
+        :type plugin_name:
+
+        :returns: time
+        :rtype: np.array
         """
         return self._dynasaur_definitions.get_ids_from_name(object_name, self._name, plugin_name)
 
     def get_ids(self):
         """
-        :return: ids as numpy  int array
+        getter
+
+        :return: ids as numpy array (np.uint32)
+        :rtype: np.array
         """
         return self._ids.astype(np.uint32)
 
     def _interp_time(self, time_):
         """
-        inteprolate time to decimal values
+        method to interpolate the given time to decimal values
+
         :param time_:
-        :return: inteprolated time vector
+        :type np.array:
+
+        :returns: interpolated time
+        :rtype: np.array as column vector
         """
 
         if self._dynasaur_definitions.get_units().second() == 1000:  # means ms
@@ -80,26 +100,128 @@ class BinoutData(object):
         # https://stackoverflow.com/questions/3844801/check-if-all-elements-in-a-list-are-identical
         # https://numpy.org/doc/stable/reference/generated/numpy.allclose.html
         time_diff = np.diff(time_)
-        if (np.allclose(time_diff, np.tile([time_diff[0]], len(time_diff)))):
+        if np.allclose(time_diff, np.tile([time_diff[0]], len(time_diff))):
             return time_
 
         time_interp = np.linspace(time_[0], end_time_, len(time_))
         return time_interp
 
-    def _read_data_types(self, elem_name=None):
+    def _set_required_data_channel_ids(self):
         """
-        :param elem_name:
+         Function to initialize member self._required_data_channel_ids
+          --> extracts the required channel_types from the calc_proc.def  (_criteria, _data_vis)
+          i.e. calc_proc.def : "array": ["(0, time)"],
+          self._required_data_channel_ids would be appended ["(0, time)"]
 
-        initialized data types
+        """
+        for def_key in self._dynasaur_definitions.__dict__:
+            if def_key == "_criteria":
+                for key in self._dynasaur_definitions.__dict__[def_key].keys():
+                    self._get_data_from_json(self._dynasaur_definitions.__dict__[def_key][key])
+            if def_key == "_data_vis":
+                for key in self._dynasaur_definitions.__dict__[def_key].keys():
+                    self._get_data_from_json(self._dynasaur_definitions.__dict__[def_key][key]["x"])
+                    self._get_data_from_json(self._dynasaur_definitions.__dict__[def_key][key]["y"])
+
+    def _get_data_from_json(self, json_object):
+        """
+        recursive method to extract array
+
+        inner part of the json object can be :
+            * value
+            * strain_stress
+            * array
+        """
+        if 'function' in json_object.keys():  # expected name and params
+            parameter_def = json_object['function']['param']
+            for key in parameter_def.keys():
+                if type(parameter_def[key]) is dict:  # step into recursion
+                    self._get_data_from_json(parameter_def[key])
+
+        else:  # data to obtain
+            if "value" in json_object.keys():
+                return
+            elif "strain_stress" in json_object.keys():
+                return
+            elif "array" in json_object.keys():
+                self._required_data_channel_ids.append(json_object['array'])
+                return
+            else:
+                assert False
+
+    def _clean_required_data_channel_ids(self):
+        """
+        clean required_data_channel_ids types, stripes each entry in self._required_data_channel_ids
+        calc_proc.def : ["(0, axial)"] --> ["axial"]
+        
+        self._required_data_channel_ids is a casted to a set
+        
+        :param: None
+        :return: None
+        """
+        _list_temp = []
+        for array_temp in self._required_data_channel_ids:
+            for type in array_temp:
+                type = type.replace("(", "")
+                type = type.replace(")", "")
+                first_type = type.split(",")[0]
+                second_type = type.split(",")[1]
+                second_type = second_type.replace(" ", "")
+                if first_type == "0" and second_type != "0":
+                    if second_type in self.DATA_CHANNEL_TYPES:
+                        _list_temp.append(second_type)
+                elif first_type != "0" and second_type == '0':
+                    if first_type in self.DATA_CHANNEL_TYPES:
+                        _list_temp.append(first_type)
+                else:
+                    if first_type in self.DATA_CHANNEL_TYPES:
+                        _list_temp.append(first_type)
+                    if second_type in self.DATA_CHANNEL_TYPES:
+                        _list_temp.append(second_type)
+        self._required_data_channel_ids = set(_list_temp)
+
+    def _get_subdatatypes_to_reach_binout_data(self, ls, ls_subdatatypes):
+        """
+        Recursively traverses binout data and stores the required
+        subdatatypes to access data in  ls_subdatatypes
+
+        i.e. binout.read("bndout", "velocity", "rigidbody") --> ["ids", "x_force"]
+          ls_subdatatypes --> [["bndout", "velocity", "rigidbody"]]
+
+        :param ls:
+            list of subdatatypes
+        :param ls_subdatatypes:
+            list of lists of subdatatyoes
+        :return: None
+        """
+        elements = self._binout.read(*ls)
+        if self.__class__.ids_name in elements:
+            ls_subdatatypes.append(ls)
+            return
+
+        for elem in elements:
+            ls_copy = copy.copy(ls)
+            ls_copy.extend([elem])
+            self._get_subdatatypes_to_reach_binout_data(ls_copy, ls_subdatatypes)
+
+    def _read_data_channels(self, subdatatypes=[]):
+        """
+        extracts the data_channels (conjunction of self.__class__.DATA_CHANNEL_TYPES and available in binout) and
+        stores the interpolated data in self._data
+
+        :param subdatatypes: subdatatypes to access data
+        :type subdatatypes: list
+
+        :returns None:
+        :rtype None:
         """
         data = {}
-        args_ = [self._name, elem_name] if elem_name is not None else [self._name]
+        args_ = subdatatypes
 
         array = []
         insert_indices = np.array([], dtype=int)
         ind = np.array([], dtype=int)
 
-        # check if type_data has to be extended due to deleted elements
         args_temp = args_ + [self.ids_name]
         ids = self._binout.read(*args_temp)
 
@@ -159,106 +281,46 @@ class BinoutData(object):
 
         self._data = {**self._data, **data}
 
-    def __init_data__(self, elem_name=None):
+    def __init_data__(self, subdatatypes=[]):
         """
-        initialized
-        :param elem_name:  only necessary for LS-dyna elout data
-        """
-        if elem_name is not None and self._name=="elout":
-            self._time = self._binout.read(self._name, elem_name, 'time').flatten()
-            self._time_interp = self._interp_time(self._time)
-            self._ids = np.array(self._binout.read(self._name, elem_name, self.__class__.ids_name)[0])
-            try:
-                len(self._ids)
-            except TypeError:
-                # case if beam data sequence consists only of one element!
-                self._ids = np.array([self._ids])
-            self._read_data_types(elem_name=elem_name)
-        else:
-            if self._name == "jntforc":
-                self._time = self._binout.read(self._name, elem_name, 'time').flatten()
-                self._time_interp = self._interp_time(self._time)
-                self._ids = np.array(self._binout.read(self._name, elem_name, self.__class__.ids_name))
-                self._read_data_types(elem_name=elem_name)
-            else:
-                self._time = self._binout.read(self._name, 'time').flatten()
-                self._time_interp = self._interp_time(self._time)
-            #    TODO: Check if data is not there index out of range -> resulting crash
-                self._ids = np.array(self._binout.read(self._name, self.__class__.ids_name))
-                self._read_data_types()
+        initialize members:
+            * self._time
+            * self._time_interp
+            * self._ids
 
-        assert (len(self._ids.shape) == 1)
+        :param subdatatypes: subdatatypes to access data in lasso.dyna.Binout
+        :type subdatatypes:  list identifiers i.e. ["bndout", "velocity", "rigidbody"]]
+        :returns: None
+        :rtype: None
+        """
 
-    def _set_defined_types(self, dynasaur_definitions):
-        """
-        set defined types
-        :param dynasaur definitions
-        """
-        for def_key in dynasaur_definitions.__dict__:
-            if def_key == "_criteria":
-                for key in dynasaur_definitions.__dict__[def_key].keys():
-                    self._get_data_from_json(dynasaur_definitions.__dict__[def_key][key])
-            if def_key == "_data_vis":
-                for key in dynasaur_definitions.__dict__[def_key].keys():
-                    self._get_data_from_json(dynasaur_definitions.__dict__[def_key][key]["x"])
-                    self._get_data_from_json(dynasaur_definitions.__dict__[def_key][key]["y"])
+        assert self._ids is None and self._time is None and self._time_interp is None
 
-    def _get_data_from_json(self, json_object):
-        """
-        recursive function
-        inner part of the json object can be :
-            * value
-            * strain_stress
-            * array
-        """
-        if 'function' in json_object.keys():  # expected name and params
-            parameter_def = json_object['function']['param']
-            for key in parameter_def.keys():
-                if type(parameter_def[key]) is dict:  # step into recursion
-                    self._get_data_from_json(parameter_def[key])
+        self._time = self._binout.read(*subdatatypes, 'time').flatten()
+        self._time_interp = self._interp_time(self._time)
+        # TODO: Check if data is not there index out of range -> resulting crash
+        self._ids = np.array(self._binout.read(*subdatatypes, self.__class__.ids_name))
+        if len(self._ids.shape) > 1:
+            self._ids = self._ids[0]
 
+        self._read_data_channels(subdatatypes=subdatatypes)
 
-        else:  # data to obtain
-            if "value" in json_object.keys():
-                return
-            elif "strain_stress" in json_object.keys():
-                return
-            elif "array" in json_object.keys():
-                self._defined_types.append(json_object['array'])
-                return
-            else:
-                print(json_object)
-                assert False
-
-    def _clean_defined_types(self):
-        """
-        clean defined types
-        :param:
-        """
-        _list_temp = []
-        for array_temp in self._defined_types:
-            for type in array_temp:
-                type = type.replace("(", "")
-                type = type.replace(")", "")
-                first_type = type.split(",")[0]
-                second_type = type.split(",")[1]
-                second_type = second_type.replace(" ", "")
-                if first_type == "0" and second_type != "0":
-                    if second_type in self.DATA_CHANNEL_TYPES:
-                        _list_temp.append(second_type)
-                elif first_type != "0" and second_type == '0':
-                    if first_type in self.DATA_CHANNEL_TYPES:
-                        _list_temp.append(first_type)
-                else:
-                    if first_type in self.DATA_CHANNEL_TYPES:
-                        _list_temp.append(first_type)
-                    if second_type in self.DATA_CHANNEL_TYPES:
-                        _list_temp.append(second_type)
-        self._defined_types = set(_list_temp)
+        assert len(self._ids.shape) == 1
 
     def read_binout_data(self):
         """
-        :return: reading success (True when data was read in, False when no data available)
+        called from plugin classes (Controller) to initialize all BinoutData objects of the DataContainer (i.e. Glstat)
+
+        __init_data__()
+        * self._time
+        * self._time_interp
+        * self._ids
+
+        read in the data channels as defined in __class__.DATA_CHANNEL_TYPES
+        _read_data_channels()
+
+        :returns: success on reading (True when data was read in, False when no data available)
+        :rtype: boolean value
         """
         # data already read
         if self._ids is not None:
@@ -272,133 +334,48 @@ class BinoutData(object):
         # data read
         self._logger.emit(LOGConstants.READ_BINOUT[0], 'read ' + self._name + ' data ...')
 
-        elements = self._binout.read(self._name)
-        elem_name = None
-        if self._name == "elout":
-            elem_name = "beam"
-            elements = self._binout.read(self._name, elem_name)
+        ls = [self._name]
+        ls_subdatatypes = []
+        self._get_subdatatypes_to_reach_binout_data(ls, ls_subdatatypes)
 
-        if self._name == "jntforc":
-            elem_name = "joints"
-            elements = self._binout.read(self._name, elem_name)
+        # go through the extracted subdatatypes and initialize what has been found
+        #   cases in where ids are in multiple subtrees are currently not supported (test case jntforces)
+        for subdatatypes in ls_subdatatypes:
+            data_channel_ids = self._binout.read(*subdatatypes)
+            negative_intersection = self._get_negative_intersection(data_channel_ids)
+            if len(negative_intersection):
+                self._logger.emit(LOGConstants.WARNING[0], "binout keys: " + " ".join(data_channel_ids))
+                self._logger.emit(LOGConstants.WARNING[0],
+                                  "Your definition file tries to access the following undefined keys : "
+                                  + " ".join(negative_intersection))
 
-        # check if required data is available (defined in SBTout.__DATA_CHANNEL_TYPES__)
-        if not self._check_negative_intersection(elements):
-            return False
+            self.__init_data__(subdatatypes)
 
-        self.__init_data__(elem_name)
         self._logger.emit(LOGConstants.READ_BINOUT[0], "done reading " + self._name + "!")
 
         return True
 
-    def _check_for_id_in_binout(self):
+    def _get_negative_intersection(self, data_channel_ids):
         """
-        check if defined id is in binout
-        :return:
-        """
-        return_function = [None]
-        if self._name == "secforc":
-            return_function = self._check_id_in_binout("_cross_sections")
-        elif self._name == "nodout":
-            return_function = self._check_id_in_binout("_node")
-        elif self._name == "jntforc":
-            return_function = self._check_id_in_binout("_jntforc")
-        elif self._name == "rcforc":
-            return_function = self._check_id_in_binout("_contact")
-        elif self._name == "deforc":
-            return_function = self._check_id_in_binout("_discrete")
-        elif self._name == "elout":
-            return_function = self._check_id_in_binout("_element")
-        elif self._name == "sbtout":
-            return_function = self._check_id_in_binout("_seatbelt")
-        elif self._name == "matsum":
-            return_function = self._check_id_in_binout("_energy_part")
-        elif self._name == "disbout":
-            return_function = self._check_id_in_binout("_disbout")
-        elif self._name == "disbout_part":
-            return_function = self._check_id_in_binout("_disbout_part")
-        elif self._name == "glstat":
-            return_function = self._check_id_in_binout("_energy_global")
-        elif self._name == "abstat":
-            return_function = self._check_id_in_binout("_abstat")
-        elif self._name == "abstat_cpm":
-            return_function = self._check_id_in_binout("_abstat_cpm")
-        elif self._name == "sleout":
-            return_function = self._check_id_in_binout("_sleout")
-        if not return_function[0]:
-            self._logger.emit(LOGConstants.WARNING[0], "Not valid ID: " + return_function[1] +
-                              ". Plese check your def file!")
+        compares the required_data_channel_ids (as defined in the calc_proc.def) with the available
+        data_channel_ids of the given binout. returns a list of missing data_channel_ids
 
-    def _check_id_in_binout(self, binout_name):
-        """
-        :param: binout name
+        :param data_channel_ids:
+        :type data_channel_ids:
 
-        :return: True or False
-        """
-        # TODO: Maybe check if the key is 0(null)
-        if binout_name == "_energy_global":
-            return True, "temp"
-        ids_list = []
-        dyn_def_key = getattr(self._dynasaur_definitions, binout_name)
-        if len(dyn_def_key) == 0:
-            return True, "temp"
-        if self._name == "elout":
-            return True, "temp"
-            # keys = self._binout.read(self._name)
-            # ids_list = np.concatenate([self._binout.read(self._name, elem_name, "ids")[0] if elem_name != "solid" else
-            #                           self._binout.read(self._name, elem_name, "ids") for elem_name in keys])
-        elif self._name == "jntforc":
-            ids_list = self._binout.read(self._name, "joints", self.ids_name)
-
-        else:
-            ids_list = self._binout.read(self._name, self.ids_name)
-
-        for sec in dyn_def_key:
-            temp_list = list(
-                map(functools.partial(self._map_help_function, binout_name, ids_list), dyn_def_key[sec]))
-
-            if len(temp_list) == 0:
-                return False, sec
-            dyn_def_key[sec] = [tmp for tmp in temp_list if tmp != None].copy()
-        return True, "temp"
-
-
-    def _map_help_function(self, binout_name, ids_list, id_):
-        """
-
-        :param binout_name:
-        :param ids_list:
-        :param id_:
-        :return:
-        """
-        if binout_name == TestConstants.CONTACT and isinstance(self._binout, lasso.dyna.Binout):
-            master_slave = id_[-1]
-            id_for_loop = int(id_[:-1])
-            if id_for_loop in ids_list:
-                return str(id_for_loop) + master_slave
-        elif id_ in ids_list:
-            return id_
-
-        return None
-
-    def _check_negative_intersection(self, elements):
-        """
-        :param: elements
-
-        :return: True or False
+        :returns negative_intersection: list of missing data_channel_ids
+        :rtype: list
         """
         # only necessary functions
-        negative_intersection = [val for val in self._defined_types if val not in elements]
-        if len(negative_intersection) != 0:
-            self._logger.emit(LOGConstants.WARNING[0], "binout keys: " + " ".join(elements))
-            self._logger.emit(LOGConstants.WARNING[0],
-                              "Your definition file tries to access the following undefined keys : " + " ".join(negative_intersection))
-        return True
+        negative_intersection = [val for val in self._required_data_channel_ids if val not in data_channel_ids]
+        return negative_intersection
 
     def get_data_of_defined_json(self, json_object, data_offsets):
         """
-        :param: json object
-        :param: data offsets
+        :param json_object:
+        :type json_object:
+        :param data_offsets:
+        :type data_offsets:
 
         :return: data array between data offset and date delta t
         """
@@ -408,7 +385,6 @@ class BinoutData(object):
             if "ID" in json_object:
                 self._logger.emit(LOGConstants.ERROR[0],
                                   "Missing ID in binary input data, identifier: " + json_object["ID"])
-
             return
 
         data_offset = 0
@@ -443,14 +419,20 @@ class BinoutData(object):
 
     def get_measurement_channel(self, id_, channel_name):
         """
-        returns the interpolated channel,
-        Aim: Constant time interval
-        :param id:
+        Returns the interpolated measurement_channel, defined through
+          - id_             (identifier in ids.def files)
+          - channel_name    ("array" in calc_proc.def )  Aim: Constant time interval for filter operations.
+
+        :param id_:
+        :type id_: str_ or int:
         :param channel_name:
-        :return:
+        :type channel_name: str
+        :returns: the interpolated channel
+        :rtype: np.array
         """
         if channel_name == 'time':
             return self._time_interp.reshape(-1, 1)
+
         if np.issubdtype(self._ids.dtype, np.number):
             data_index = np.where(int(id_) == self._ids)[0]
         else:  # madymo case ... ids are not numeric
@@ -467,6 +449,6 @@ class BinoutData(object):
             self._logger.emit(LOGConstants.ERROR[0], str(id_) + ' has no data with the identifier : ' + channel_name)
             return []
 
-        d = self._data[channel_name][:, data_index]
+        interpolated_channel = self._data[channel_name][:, data_index]
 
-        return d.reshape(-1, len(data_index))
+        return interpolated_channel.reshape(-1, len(data_index))
