@@ -1,12 +1,51 @@
 import time
 import json
+from lasso.dyna.Binout import Binout
+
 
 from ..utils.constants import LOGConstants, DefFileConstants, UnitsConstants, DefinitionConstants, \
-    ObjectConstantsForData, JsonConstants, LoggerERROR, LoggerWARNING, PluginsParamDictDef
+    ObjectConstantsForData, JsonConstants, LoggerERROR, LoggerWARNING, PluginsParamDictDef, DataTypeMapping
 from ..utils.constants import LoggerReadDynasaurDefinitionFile as LD
 from ..data.def_file_validator import TestDefinitionJSON
 
 
+class Helpers(object):
+
+    @staticmethod
+    def create_default_object_definitions(data_source, code_type):
+        """
+        TODO
+        documentation
+
+        :param data_source:
+        :param code_type:
+        :return:
+        """
+        d = {"OBJECTS":[]}
+        if code_type == "LS-DYNA":
+            binout = Binout(data_source)
+            for data_type in binout.read():
+                if "ids" in binout.read(data_type):
+                    legend_ids = [binout.read(data_type, "legend")[i:i + 80].strip(" ") for i in
+                                  range(0, len(binout.read(data_type, "legend")), 80)]
+                    if len(binout.read(data_type, "ids").shape) > 1:
+                        ids = binout.read(data_type, "ids")[0]
+                    else:
+                        ids = binout.read(data_type, "ids")
+                    for nr, id in enumerate(ids):
+                        dynasaur_identifier = DataTypeMapping.binary_identifier_2_dynasaur_identifier(data_type)
+                        if dynasaur_identifier is None:
+                            print(data_type)
+                            break
+                        if len(legend_ids) == len(binout.read(data_type, "ids")):
+                            d["OBJECTS"].append({"type": dynasaur_identifier, "name": legend_ids[nr], "id": [int(id)]})
+                        else:
+                            d["OBJECTS"].append({"type": dynasaur_identifier, "name": data_type + str(nr), "id": [int(id)]})
+
+        return d
+
+
+
 class Units(object):
     def __init__(self):
         self._time = 1.0  # in seconds
@@ -75,27 +114,21 @@ class DynasaurDefinitions:
         self._info = []
 
         # set all class members
-        self._cross_sections = {}
-        self._node = {}
-        self._jntforc = {}
-        self._disbout = {}
-        self._part_disbout = {}
-        self._contact = {}
-        self._discrete = {}
-        self._element = {}
-        self._seatbelt = {}
-        self._energy_part = {}
-        self._energy_global = {}
-        self._abstat = {}
-        self._sleout = {}
-        self._abstat_cpm = {}
-        self._riskfunction = {DefFileConstants.FUNC_CRITERIA: {},
-                              DefFileConstants.FUNC_DATA_VISUALIZATION: {}}
+        self._dynasaur_data_type_container = {ObjectConstantsForData.__dict__[i]: {}
+                                              for i in ObjectConstantsForData.__dict__ if not i.startswith("__")}
 
         self._criteria = {}
         self._data_vis = {}
         self._units = Units()
 
+    def binout_type_data_type_container(self, binout_name):
+        """
+
+        :param binout_name:
+        :return:
+        """
+        return DataTypeMapping.binary_identifier_2_dynasaur_identifier(binout_name) in self._dynasaur_data_type_container
+
     def get_param_dict_from_command(self, command):
         """
         :param command:
@@ -145,7 +178,7 @@ class DynasaurDefinitions:
         :return:
         """
 
-        ls = [key  for key in self._data_vis.keys()]
+        ls = [key for key in self._data_vis.keys()]
         ret = []
         for key in ls:
             d = {"visualization": key}
@@ -160,7 +193,7 @@ class DynasaurDefinitions:
         return ret
 
     def get_criteria_calc_commands(self):
-        ls = [key for key in self._data_vis.keys()]
+        ls = [key for key in self._criteria.keys()]
         ret = []
         for nr, key in enumerate(ls):
             ret.append({"criteria": key})
@@ -188,35 +221,8 @@ class DynasaurDefinitions:
         :param plugin_name:
         :return dict(name):
         """
-        d = {}
-        if data_container == "nodout":
-            d = self._node
-        elif data_container == "jntforc":
-            d = self._jntforc
-        elif data_container == "rcforc":
-            d = self._contact
-        elif data_container == "deforc":
-            d = self._discrete
-        elif data_container == "elout":
-            d = self._element
-        elif data_container == "secforc":
-            d = self._cross_sections
-        elif data_container == "sbtout":
-            d = self._seatbelt
-        elif data_container == "matsum":
-            d = self._energy_part
-        elif data_container == "glstat":
-            d = self._energy_global
-        elif data_container == "disbout":
-            d = self._disbout
-        elif data_container == "disbout_part":
-            d = self._part_disbout
-        elif data_container == "abstat":
-            d = self._abstat
-        elif data_container == "abstat_cpm":
-            d = self._abstat_cpm
-        elif data_container == "sleout":
-            d = self._sleout
+        dynasaur_data_type = DataTypeMapping.binary_identifier_2_dynasaur_identifier(data_container)
+        d = self._dynasaur_data_type_container[dynasaur_data_type]
 
         if name not in d.keys() or len(d) == 0:
             self._logger.emit(LOGConstants.ERROR[0],
@@ -348,14 +354,20 @@ class DynasaurDefinitions:
         name = json_object[JsonConstants.NAME]
 
         if JsonConstants.ID in json_object:
-            self._contact[name] = json_object[JsonConstants.ID]
+            self._dynasaur_data_type_container["CONTACT"][name] = json_object[JsonConstants.ID]
 
         if JsonConstants.ID_RANGE in json_object:
-            self._contact[name] = [str(i) + json_object[JsonConstants.ID_RANGE][0][-1]
+            self._dynasaur_data_type_container["CONTACT"][name] = [str(i) + json_object[JsonConstants.ID_RANGE][0][-1]
                                    for i in range(int(json_object[JsonConstants.ID_RANGE][0][:-1]),
                                                   int(json_object[JsonConstants.ID_RANGE][-1][:-1]) + 1)]
 
     def _set_datacontainer_defintion(self, json_object, def_container):
+        """
+
+        :param json_object:
+        :param def_container:
+        :return:
+        """
         name = json_object[JsonConstants.NAME]
 
         if JsonConstants.ID in json_object:
@@ -403,39 +415,18 @@ class DynasaurDefinitions:
         """
         for json_object in json_objects[DefinitionConstants.OBJECTS]:
 
-            if json_object[JsonConstants.TYPE] == ObjectConstantsForData.ELEMENTOBJECT:
-                self.__set_object(json_object)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.NODE:
-                self._set_datacontainer_defintion(json_object, self._node)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.JOINT:
-                self._set_datacontainer_defintion(json_object, self._jntforc)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.AIRBAG:
-                self._set_datacontainer_defintion(json_object, self._abstat)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.AIRBAG_CPM:
-                self._set_datacontainer_defintion(json_object, self._abstat_cpm)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.DISBOUT:
-                self._set_datacontainer_defintion(json_object, self._disbout)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.ELEMENT:
-                self._set_datacontainer_defintion(json_object, self._element)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.CONTACT:
-                self._set_contact(json_object)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.DISCRETE:
-                self._set_datacontainer_defintion(json_object, self._discrete)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.SEAT_BELT:
-                self._set_datacontainer_defintion(json_object, self._seatbelt)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.CROSS_SECTION:
-                self._set_datacontainer_defintion(json_object, self._cross_sections)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.ENERGY_PART:
-                self._set_datacontainer_defintion(json_object, self._energy_part)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.ENERGY_GLOBAL:
-                self._set_datacontainer_defintion(json_object, self._energy_global)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.DISBOUT_PART:
-                self._set_datacontainer_defintion(json_object, self._part_disbout)
-            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.SLEOUT:
-                self._set_datacontainer_defintion(json_object, self._sleout)
-            else:
+            if json_object[JsonConstants.TYPE] not in self._dynasaur_data_type_container.keys():
                 self._logger.emit(LOGConstants.WARNING[0],
                                   json_object[JsonConstants.TYPE] + LoggerWARNING.print_statements[1])
+                continue
+
+            if json_object[JsonConstants.TYPE] == ObjectConstantsForData.ELEMENTOBJECT:
+                self.__set_object(json_object)
+            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.CONTACT:
+                self._set_contact(json_object)
+            else:
+                data_type_container = self._dynasaur_data_type_container[json_object[JsonConstants.TYPE]]
+                self._set_datacontainer_defintion(json_object, data_type_container)
 
     def _set_unit(self, json_object):
         """
@@ -520,77 +511,64 @@ class DynasaurDefinitions:
         """
         if fn is None:
             exit("No definition file")
+
         if isinstance(fn, str):
-            self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.READ % fn)
-            with open(fn) as fd:
-                try:
-                    json_objects = json.load(fd)
-                except ValueError as e:
-                    self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[4] % e)
-                    exit("Exiting - Invalid JSON file")  # Because Mutant, if GUI "exit()" comment
-                    return
-        elif isinstance(fn, list):
+            fn = [fn]
+
+        if isinstance(fn, list):
+
             if len(fn) < 1:
                 exit("No definition file(s) in list")
-            else:
-                first = True
-                for single_def_file in fn:
+            is_entity_set = {i: False for i in DefinitionConstants.__dict__ if i[:1] != "_"}
+
+            for single_def_file in fn:
+                self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.READ % single_def_file)
+                try:
                     with open(single_def_file) as fd:
                         try:
-                            if first:
-                                json_objects = json.load(fd)
-                                first = False
-                                object_order = []
-                                for json_object in json_objects:
-                                    if DefinitionConstants.TITLE in json_object:
-                                        object_order.append(DefinitionConstants.TITLE)
-                                    elif DefinitionConstants.UNIT in json_object:
-                                        object_order.append(DefinitionConstants.UNIT)
-                                    elif DefinitionConstants.CRITERIA in json_object:
-                                        object_order.append(DefinitionConstants.CRITERIA)
-                                    elif DefinitionConstants.DATA_VIS in json_object:
-                                        object_order.append(DefinitionConstants.DATA_VIS)
-                                    elif DefinitionConstants.OBJECTS in json_object:
-                                        object_order.append(DefinitionConstants.OBJECTS)
-                            else:
-                                new_json_file = json.load(fd)
-                                for new_json_object in new_json_file:
-                                    key = list(new_json_object.keys())[0]
-                                    if key == DefinitionConstants.TITLE or key == DefinitionConstants.UNIT:
-                                        if json_objects[object_order.index(key)][key] != new_json_object[key]:
-                                            if key == DefinitionConstants.TITLE:
-                                                exit("Exiting - Different def-file titles!")
-                                            elif key == DefinitionConstants.UNIT:
-                                                exit("Exiting - Different def-file units!")
-                                    elif key == DefinitionConstants.CRITERIA or key == DefinitionConstants.DATA_VIS or key == DefinitionConstants.OBJECTS:
-                                        json_objects[object_order.index(key)][key] = json_objects[object_order.index(key)][key] + new_json_object[key]
+                            json_objects = json.load(fd)
+                            for j_object in json_objects:
+                                if DefinitionConstants.TITLE in j_object:
+                                    if not is_entity_set[DefinitionConstants.TITLE]:
+                                        self._title = j_object[DefinitionConstants.TITLE]
+                                        is_entity_set[DefinitionConstants.TITLE] = True
                                     else:
-                                        self._logger.emit(LOGConstants.ERROR[0], "json object not understood:" + str(json_object))
-                                        return
+                                        self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[8] + DefinitionConstants.TITLE)
+                                        exit("Entity Title already set " + self._title)
+
+                                elif DefinitionConstants.UNIT in j_object:
+                                    if not is_entity_set[DefinitionConstants.UNIT]:
+                                        self._set_unit(j_object)
+                                        is_entity_set[DefinitionConstants.UNIT] = True
+                                    else:
+                                        self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[8] + DefinitionConstants.UNIT)
+                                        exit("Entity Units already set " + self._units)
+
+                                elif DefinitionConstants.CRITERIA in j_object:
+                                    self._set_criteria(j_object)
+
+                                elif DefinitionConstants.DATA_VIS in j_object:
+                                    self._set_diagram_visualisation(j_object)
+                                    self._set_info()
+
+                                elif DefinitionConstants.OBJECTS in j_object:
+                                    self._parse_objects(json_objects)
+
+                                else:
+                                    self._logger.emit(LOGConstants.ERROR[0],
+                                                      "json object not understood:" + str(j_object))
+                                    return
+
                         except ValueError as e:
                             self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[4] % e)
                             exit("Exiting - Invalid JSON file")  # Because Mutant, if GUI "exit()" comment
                             return
+                except FileNotFoundError as e:
+                    self._logger.emit(LOGConstants.ERROR[0],  e.strerror + " " + e.filename)
+                    exit(e.strerror + " " + e.filename)
+
         else:
             exit("No definition file")
 
-        # TestDefinitionJSON.test_def_json(json_objects, self._logger)
-
-        for j_object in json_objects:
-            if DefinitionConstants.TITLE in j_object:
-                self._title = j_object[DefinitionConstants.TITLE]
-            elif DefinitionConstants.UNIT in j_object:
-                self._set_unit(j_object)
-            elif DefinitionConstants.CRITERIA in j_object:
-                self._set_criteria(j_object)
-            elif DefinitionConstants.DATA_VIS in j_object:
-                self._set_diagram_visualisation(j_object)
-                self._set_info()
-            elif DefinitionConstants.OBJECTS in j_object:
-                self._parse_objects(json_objects)
-            else:
-                self._logger.emit(LOGConstants.ERROR[0], "json object not understood:" + str(j_object))
-                return
-
         self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.DONE)
 
