@@ -30,6 +30,45 @@ class DEForc(BinoutData):
         BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'deforc')
 
 
+
+class RBDout(BinoutData):
+    """
+    """
+    DATA_CHANNEL_TYPES = ['x_force', 'y_force', 'z_force', 'displacement', 'resultant_force']
+    ids_name = "ids"
+
+    def __init__(self, binout, logger, dynasaur_definitions):
+        """
+        Initialization i.e. call BinoutData constructor
+
+        :param: binout
+        :param: logger
+        :param: dynasaur definition
+
+        :return:
+        """
+        BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'rbdout')
+
+
+class BDNout(BinoutData):
+    """
+    """
+    DATA_CHANNEL_TYPES = ['x_force', 'y_force', 'z_force', 'displacement', 'resultant_force']
+    ids_name = "ids"
+
+    def __init__(self, binout, logger, dynasaur_definitions):
+        """
+        Initialization i.e. call BinoutData constructor
+
+        :param: binout
+        :param: logger
+        :param: dynasaur definition
+
+        :return:
+        """
+        BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'bdnout')
+
+
 class Disbout(BinoutData):
     DATA_CHANNEL_TYPES = ['r_dis_axial', 'moment_s', 'rslt_nt', 'r_dis_nt', 't_dir_y', 'axial_rot',
                           's_dir_y', 'r_dis_ns', 'matid', 's_dir_x', 'torsion', 'rslt_ns', 'axial_z', 'mtype',
@@ -504,7 +543,7 @@ class EloutObject(BinoutData):
 
         return (np.concatenate((ids, mat, ipts), axis=1)).astype(int)
 
-    def _indices_of_destroyed_elements(self, elout_elem, type):
+    def _indices_of_destroyed_elements(self, elout_elem, type_):
         """
         indices list of destroyed elements over time
         used to insert dummy values [used for padding to full sized matrix
@@ -520,7 +559,7 @@ class EloutObject(BinoutData):
             return 1.) insert_index
 
         :param elout_elem:
-        :param type:
+        :param type_:
         :return:
         """
 
@@ -531,14 +570,14 @@ class EloutObject(BinoutData):
         ids = self._binout.read(self._name, elout_elem, 'ids')
 
         # check if elements have been destroyed
-        if isinstance(ids[0], np.intp): # or isinstance(ids[0], np.int64):
+        if np.issubdtype(type(ids[0]), np.integer): # or isinstance(ids[0], np.int64):
             ids = [np.array([i]) for i in ids]
         if sum([len(i) - len(ids[0]) for i in ids]) == 0:
             return insert_indices, ind
 
         # if stress and shell, there might be various integration points:
         #    the read ids have to be extended accordingly
-        if type == "stress" and elout_elem == "shell" or type == "stress" and elout_elem == "beam":
+        if type_ == "stress" and elout_elem == "shell" or type_ == "stress" and elout_elem == "beam":
             nip = self._binout.read(self._name, elout_elem, 'nip')
             ids_nip = []
             for i in range(0, ids.shape[0]):
@@ -973,6 +1012,31 @@ class Elout(BinoutData):
         """
         BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'elout')
 
+    def __init_data__(self, subdatatypes=[]):
+        """
+
+        :param elem_name:
+        :return:
+        """
+        self._time = self._binout.read(*subdatatypes, 'time').flatten()
+        self._time_interp = self._interp_time(self._time)
+        self._ids = np.array(self._binout.read(*subdatatypes, self.__class__.ids_name)[0])
+        try:
+            len(self._ids)
+        except TypeError:
+            # case if beam data sequence consists only of one element!
+            self._ids = np.array([self._ids])
+        self._read_data_channels(subdatatypes=subdatatypes)
+
+    def _get_subdatatypes_to_reach_binout_data(self, ls, ls_subdatatypes):
+        """
+
+        :param ls: 
+        :param ls_subdatatypes: 
+        :return: 
+        """
+        ls_subdatatypes.append([self._name, 'beam'])
+
 
 class Abstat(BinoutData):
     # applies for beam elements!
@@ -1038,7 +1102,16 @@ class Glstat(BinoutData):
         """
         BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'glstat')
 
-    def _read_data_types(self):
+    def _get_subdatatypes_to_reach_binout_data(self, ls, ls_subdatatypes):
+        """
+
+        :param ls:
+        :param ls_subdatatypes:
+        :return:
+        """
+        ls_subdatatypes.append([ls[0]])
+
+    def _read_data_channels(self, subdatatypes=[]):
         """
         initialisation of self._data
 
@@ -1088,10 +1161,11 @@ class Matsum(BinoutData):
     DATA_CHANNEL_TYPES = ['x_momentum', 'y_rbvelocity', 'z_rbvelocity', 'internal_energy', 'kinetic_energy',
                           'max_shell_mass', 'y_momentum', 'mass', 'eroded_kinetic_energy', 'brick_id', 'max_brick_mass',
                           'shell_id', 'eroded_internal_energy', 'z_momentum', 'hourglass_energy',
-                          'eroded_hourglass_energy' 'x_rbvelocity','energy_ratio','eroded_energy_ratio','x_rbacceleration',
-                          'y_acceleration','z_acceleration','x_rbdisplacement','y_rbdisplacement','z_rbdisplacement',
-                          'resultant_displacement','resultant_velocity','resultant_acceleration','resultant_momentum',
-                          'x_acceleration','y_acceleration','z_acceleration']
+                          'eroded_hourglass_energy', 'x_rbvelocity', 'energy_ratio', 'eroded_energy_ratio',
+                          'x_rbacceleration', 'y_acceleration', 'z_acceleration', 'x_rbdisplacement',
+                          'y_rbdisplacement', 'z_rbdisplacement', 'resultant_displacement', 'resultant_velocity',
+                          'resultant_acceleration', 'resultant_momentum', 'x_acceleration', 'y_acceleration',
+                          'z_acceleration']
 
     ids_name = "ids"
 
@@ -1108,6 +1182,60 @@ class Matsum(BinoutData):
         BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'matsum')
 
 
+class RigidBody(BinoutData):
+    """
+    """
+    DATA_CHANNEL_TYPES = ['local_ax', 'local_ay', 'local_az', 'local_dx', 'local_dy', 'local_dz', 'local_vx',
+                          'local_vy', 'local_vz', 'local_rax', 'local_ray', 'local_raz', 'local_rdx', 'local_rdy',
+                          'local_rdz', 'local_rvx', 'local_rvy', 'local_rvz', 'global_x', 'global_y', 'global_z',
+                          'global_dx', 'global_dy', 'global_dz', 'global_ax', 'global_ay', 'global_az', 'global_vx',
+                          'global_vy', 'global_vz', 'global_rax', 'global_ray', 'global_raz', 'global_rdx',
+                          'global_rdy', 'global_rdz', 'global_rvx', 'global_rvy', 'global_rvz', 'dircos_11',
+                          'dircos_12', 'dircos_13', 'dircos_21', 'dircos_22', 'dircos_23', 'dircos_31', 'dircos_32',
+                          'dircos_33']
+    ids_name = "ids"
+
+    def __init__(self, data, logger, dynasaur_definitions):
+        """
+        Initialization/constructor
+
+        :param: binout
+        :param: logger
+        :param: dynasaur definition
+        :param: cody type(MADYMO/LS DYNA)
+
+        :return:
+        """
+
+        BinoutData.__init__(self, data, logger, dynasaur_definitions, 'rbdout')
+
+
+class BoundaryCondition(BinoutData):
+    """
+    """
+    # DATA_CHANNEL_TYPES = ['ids', 'energy', 'revision', 'y_force', 'zmoment', 'x_force', 'title', 'y_total',
+    #                       'legend', 'version', 'xmoment', 'ymoment', 'legend_ids', 'z_force', 'date', 'time',
+    #                       'etotal', 'x_total', 'z_total']
+    DATA_CHANNEL_TYPES = ['energy', 'y_force', 'x_force', 'xmoment', 'ymoment', 'zmoment',
+                          'z_force', 'etotal', 'x_total', 'y_total', 'z_total']
+
+    ids_name = "ids"
+
+    def __init__(self, data, logger, dynasaur_definitions):
+        """
+        Initialization/constructor
+
+        :param: binout
+        :param: logger
+        :param: dynasaur definition
+        :param: cody type(MADYMO/LS DYNA)
+
+        :return:
+        """
+
+        BinoutData.__init__(self, data, logger, dynasaur_definitions, 'bndout')
+
+
 class Nodout(BinoutData):
     """
     """
@@ -1115,8 +1243,8 @@ class Nodout(BinoutData):
                           'rz_displacement', 'rx_acceleration', 'ry_acceleration', 'rz_acceleration', 'x_coordinate',
                           'y_coordinate', 'z_coordinate', 'x_displacement', 'y_displacement', 'z_displacement',
                           'x_velocity', 'y_velocity', 'z_velocity', 'x_acceleration', 'y_acceleration',
-                          'z_acceleration','resultant_rotation','resultant_angular_velocity',
-                          'resultant_angular_acceleration','resultant_displacement','resultant_velocity',
+                          'z_acceleration', 'resultant_rotation', 'resultant_angular_velocity',
+                          'resultant_angular_acceleration', 'resultant_displacement', 'resultant_velocity',
                           'resultant_acceleration']
     ids_name = "ids"
 
@@ -1209,11 +1337,9 @@ class JointForc(BinoutData):
         return d.reshape(-1, len(data_index))
 
 
-
-
 class RCForc(BinoutData):
     DATA_CHANNEL_TYPES = ['x_moment', 'y_moment', 'z_moment', 'x_force', 'y_force', 'z_force', 'tie_count', 'mass',
-                          'tie_area','resultant_force']
+                          'tie_area', 'resultant_force']
 
     ids_name = "ids"
 
@@ -1348,6 +1474,3 @@ class Sleout(BinoutData):
             return self._time.reshape(-1, 1)
         d = self._data[channel_name][:, 0]
         return d
-
-
-
