@@ -101,6 +101,11 @@ class MadymoData:
                 self._d[data_type]["sliding_interface_energy"] = None
 
     def _get_simplified_rcforc_id(self, id_string):
+        """
+
+        :param id_string:
+        :return:
+        """
         id_string_ = id_string.split(" ")
         xy = [re.search("(\/[0-9]+)+", i).group(0) for i in id_string_ if re.search("(\/[0-9]+)+", i) is not None]
         if id_string.split(":")[-1].isnumeric():
@@ -139,6 +144,7 @@ class MadymoData:
             # if assert: ID is already in id list
 
             self._d[data_type]["ids"].append(simiplified_id)
+            self._d[data_type]["legend_ids"].append(id_string)
 
     def _add_time(self, node, data_type):
         """
@@ -181,7 +187,7 @@ class MadymoData:
                 a = 0
                 b = 0
                 dimension = (self._d['rcforc']['time'].shape[0], len(self._d['rcforc']['ids']))
-                data_types_to_be_filled = [i for i in self._d['rcforc'].keys() if i != "ids" and i != "time"]
+                data_types_to_be_filled = [i for i in self._d['rcforc'].keys() if i != "ids" and i != "time" and i != "legend_ids"]
                 for data_type in data_types_to_be_filled:
                     self._d['rcforc'][data_type] = np.empty(dimension)
                     self._d['rcforc'][data_type][:] = np.nan
@@ -198,7 +204,7 @@ class MadymoData:
             # extend the data dictionary
             if "nodout" in self._d:
                 dimension = (self._d['nodout']['time'].shape[0], len(self._d['nodout']['ids']))
-                data_types_to_be_filled = [i for i in self._d['nodout'].keys() if i != "ids" and i != "time"]
+                data_types_to_be_filled = [i for i in self._d['nodout'].keys() if i != "ids" and i != "time" and i != "legend_ids"]
                 for data_type in data_types_to_be_filled:
                     self._d['nodout'][data_type] = np.empty(dimension)
                     self._d['nodout'][data_type][:] = np.nan
@@ -261,7 +267,7 @@ class MadymoData:
 
             if "matsum" in self._d:
                 dimension = (self._d['matsum']['time'].shape[0], len(self._d['matsum']['ids']))
-                energy_to_be_filled = [i for i in self._d['matsum'].keys() if i != "ids" and i != "time"]
+                energy_to_be_filled = [i for i in self._d['matsum'].keys() if i != "ids" and i != "time" and i != "legend_ids"]
                 for energy_type in energy_to_be_filled:
                     self._d['matsum'][energy_type] = np.empty(dimension)
                     self._d['matsum'][energy_type][:] = np.nan
@@ -269,7 +275,7 @@ class MadymoData:
 
             if "glstat" in self._d:
                 dimension = (self._d['glstat']['time'].shape[0],len(self._d['glstat']['ids']))
-                energy_to_be_filled = [i for i in self._d['glstat'].keys() if i != "ids" and i != "time"]
+                energy_to_be_filled = [i for i in self._d['glstat'].keys() if i != "ids" and i != "time" and i != "legend_ids"]
                 for energy_type in energy_to_be_filled:
                     self._d['glstat'][energy_type] = np.empty(dimension)
                     self._d['glstat'][energy_type][:] = np.nan
@@ -333,21 +339,21 @@ class MadymoData:
                 if 'nodout' not in self._d.keys():
                     if node.parent.name.split("/")[-1].startswith("Angular ") or \
                             node.parent.name.split("/")[-1].startswith("Linear "):
-                        self._d['nodout'] = {"ids": [], "time": None}
+                        self._d['nodout'] = {"ids": [], "legend_ids": [], "time": None}
 
                 if 'glstat' not in self._d.keys():
                     if node.parent.name.split("/")[-1].startswith("Energy output"):
-                        self._d['glstat'] = {"ids": [], "time": None}
+                        self._d['glstat'] = {"ids": [], "legend_ids":[], "time": None}
 
                 if 'matsum' not in self._d.keys():
                     if node.parent.name.split("/")[-1].startswith("Energy output"):
-                        self._d['matsum'] = {"ids": [], "time": None}
+                        self._d['matsum'] = {"ids": [], "legend_ids":[], "time": None}
 
                 if 'rcforc' not in self._d.keys():
                     if node.parent.name.split("/")[-1].startswith("contact loads"):
-                        self._d['rcforc'] = {"ids": [], "time": None}
-
+                        self._d['rcforc'] = {"ids": [], "legend_ids":[], "time": None}
 
+                print(node.parent.name)
                 if node.parent.name.split("/")[-1].startswith("Energy output"):
                     # TODO
                     # self._d['glstat'][DataChannelTypesNodout.RX_ACCELERATION] = None
