@@ -15,9 +15,9 @@ class CriteriaController(Plugin, PluginInterface):
         :param: calculation_procedure_def_file
         :param: object_def_file
         :param: data_source(binout path)
-        :param: logger
-        :param: dynasaur definition
+        :param: user_function_object
         :param: volume_def_file
+        :param: code_type
 
         :return:
         """
