@@ -160,7 +160,8 @@ class Plugin(object):
             self._data = {data: DataContainer.get_data(data) for data in self._data_requirements}
             for data_name in self._data.keys():
                 if self._data[data_name] is None:
-                    self._logger.emit(LOGConstants.ERROR[0], data_name + LoggerERROR.print_statements[1])
+                    self._logger.emit(LOGConstants.ERROR[0], data_name + LoggerERROR.print_statements[1] +
+                                        ", ".join(sorted([i for i in ObjectConstantsForData.__dict__.keys() if i[:1] != '_'])))
                 if not self._data[data_name].read_binout_data():
                     self._logger.emit(LOGConstants.ERROR[0], data_name + LoggerERROR.print_statements[2])
 
