@@ -108,7 +108,6 @@ class MadymoConstants:
 
 
 class DefinitionConstants(object):
-    CODE = "CODE"
     TITLE = "TITLE"
     UNIT = "UNIT"
     OBJECTS = "OBJECTS"
@@ -148,6 +147,57 @@ class ObjectConstantsForData:
     DISBOUT = "DISBOUT"
     DISBOUT_PART = "DISBOUT_PART"
     SLEOUT = "SLEOUT"
+    RIGID_BODY = "RIGID_BODY"
+    BOUNDARY_CONDITION = "BOUNDARY_CONDITION"
+
+
+class DataTypeMapping:
+    mapping = [(ObjectConstantsForData.ELEMENT, "elout"),
+                (ObjectConstantsForData.AIRBAG, "abstat"),
+                (ObjectConstantsForData.AIRBAG_CPM, "abstat_cpm"),
+                (ObjectConstantsForData.CROSS_SECTION, "secforc"),
+                (ObjectConstantsForData.NODE, "nodout"),
+                (ObjectConstantsForData.JOINT, "jntforc"),
+                (ObjectConstantsForData.SEAT_BELT, "sbtout"),
+                (ObjectConstantsForData.CONTACT, "rcforc"),
+                (ObjectConstantsForData.DISCRETE, "deforc"),
+                (ObjectConstantsForData.ENERGY_PART, "matsum"),
+                (ObjectConstantsForData.ENERGY_GLOBAL, "glstat"),
+                (ObjectConstantsForData.DISBOUT, "disbout"),
+                (ObjectConstantsForData.DISBOUT_PART, "disbout_part"),
+                (ObjectConstantsForData.SLEOUT, "sleout"),
+                (ObjectConstantsForData.RIGID_BODY, "rbdout"),
+                (ObjectConstantsForData.BOUNDARY_CONDITION, "bndout"),
+                ]
+
+    @staticmethod
+    def binary_identifier_2_dynasaur_identifier(binary_data_type):
+        """
+
+        :param binary_data_type:
+        :return:
+        """
+
+        for tpl in DataTypeMapping.mapping:
+            if tpl[1] == binary_data_type:
+                return tpl[0]
+
+        return None
+
+
+    @staticmethod
+    def dynasaur_identifier_2_binary_identifier(dynasaur_data_type):
+        """
+
+        :param dynasaur_data_type:
+        :return:
+        """
+        for tpl in DataTypeMapping.mapping:
+            if tpl[0] == dynasaur_data_type:
+                return tpl[1]
+
+        return None
+
 
 
 class JsonConstants:
@@ -239,7 +289,6 @@ class LoggerDefinitionFileValidator:
     PLUGIN_MISSING = "Plugin has to be defined!"
     LOWER_CASE = "Definition has to be be lowercase!"
     NAME_MISSING = "Name has to be defined!"
-    DCDF_CCDF_MISSING = "Risk functions have to be defined as DCDF or CCDF!"
     OBJECT_MISSING = "Objects have to be defined in object!"
     TYPE_MISSING = "object type is missing!"
     PART_OF_MISSING = "\"part_of\" is missing! Definition name: "
@@ -254,13 +303,14 @@ class LoggerDefinitionFileValidator:
 
 class LoggerERROR:
     print_statements = {
-        1: " is not supported as data type. Use ELEMENT, CROSS_SECTION, NODE, DISCRETE, SEAT_BELT, CONTACT",
+        1: " is not supported as data type. Use ",
         2: " not available in your binout",
         3: " No file to select",
         4: " JSON object issue: %s",
         5: " Definition does not contain key ",
         6: " In CRITERIA plugin: Definition does not contain key ",
-        7: " In DATA VISUALIZATION plugin: Definition does not contain key "
+        7: " In DATA VISUALIZATION plugin: Definition does not contain key ",
+        8: " Double define of "
     }
 
 
